/*-------------------------------------------------------------------------+
| Copyright 2023 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.quality.ui.view.fx;

import static org.fortiss.tooling.kernel.ui.util.SelectionUtils.checkAndPickFirst;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getFirstParentWithType;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.common.ui.javafx.layout.CompositeFXControllerBase;
import org.fortiss.tooling.ext.quality.service.ModelQualityService;
import org.fortiss.tooling.kernel.model.IProjectRootElement;

import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.SplitPane;

/**
 * FX Controller for the Metrics view.
 *
 * @author groh
 */

@SuppressWarnings("unchecked")
public class MetricsFXController extends CompositeFXControllerBase<SplitPane, Node>
		implements ISelectionListener {

	/** The {@link IProjectRootElement} in which the selected element is contained in. */
	private IProjectRootElement currentRootElement;

	/** The refresh {@link Button} to refresh the metrics. */
	@FXML
	private Button refreshButton;

	/** The central top {@link Label} to display short messages. */
	@FXML
	private Label topLabel;

	/** Time format when displaying updating time. */
	private final DateTimeFormatter timeFormat = DateTimeFormatter.ofPattern("HH:mm:ss");

	/** {@inheritDoc} */
	@Override
	public String getFXMLLocation() {
		return "MetricsViewFx.fxml";
	}

	/** {@inheritDoc} */
	@Override
	public void initialize() {
		// Not used
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {

		ModelQualityService.getInstance();
		var last_metrics = ModelQualityService.last_metrics;
		if(last_metrics == null) {
			topLabel.setText(
					"No Metrics were available at " + timeFormat.format(LocalDateTime.now()));
			return;
		}

		EObject selected = checkAndPickFirst(selection, EObject.class);

		if(selected == null) {
			topLabel.setText("Nothing selected");
			return;
		}

		if(selected instanceof IProjectRootElement) {
			currentRootElement = (IProjectRootElement)selected;
		} else {
			currentRootElement = getFirstParentWithType(selected, IProjectRootElement.class);
		}

		if(currentRootElement == null) {
			// There was no root element for the selected element. (Usually should be the
			// FileProject.) This is not displayed here.
			topLabel.setText("No root found");
			return;
		}

		if(!(currentRootElement instanceof IHierarchicElement)) {
			topLabel.setText("Root is not an IHierarchicElement");
			return;
		}

		if(selected instanceof IHierarchicElement) {
			var root_provider = last_metrics.get(currentRootElement);
			var element = (IHierarchicElement)selected;
			var provider = root_provider.getProvider(element);
			if(provider != null) {

				var string = provider.getExampleString();
				topLabel.setText(string);
			} else {
				topLabel.setText("Element not in metrics");
			}
		} else {
			topLabel.setText("Selected is not an IHierarchicElement");
		}

	}

	/**
	 * Is triggered when the refresh button is pressed.
	 */
	public void onRefreshMetricsSubmit() {
		LocalDateTime currentTime = LocalDateTime.now();
		topLabel.setText("View was updated at " + timeFormat.format(currentTime));
	}
}
